#ifndef Python_h
#define Python_h

#define DEBUG_PYTHON

#include <Python.h>
#include <list>

#if PY_VERSION_HEX >= 0x02030000
#define PYTHON_SUPPORTS_GIL
#else
#undef PYTHON_SUPPORTS_GIL
#endif

class Python {
public:
   virtual ~Python();
public:
   bool initialize();
public:
   PyObject*     pyr             (char*        str);
   void          py              (char*        str);
public:
   void          pyf             (const char*  format, ...);
   PyObject*     pyrf            (const char*  format, ...);
   char*         pyString        (PyObject*    obj);
   PyObject*     pyCall          (char*        name,
                                  char*        format,
                                  ...);
   PyObject*     pyMethodCall    (PyObject*    self,
                                  char*        name,
                                  char*        format,
                                  ...);
   PyObject*     pyLookup        (const char*  name);
   virtual void  pyExceptExpect  (PyObject*    xcpt);
   virtual void  pyExceptClear   (void);
   virtual bool  pyExceptMatches (PyObject*    result,
                                  PyObject*    xcpt);
   PyObject*     pyExceptValue   ();
   PyObject*     pyExceptValuef  ();
   PyObject*     pyValue         (char*        dict,
                                  char*        name);
   void          pyAssign        (char*        dict,
                                  char*        name,
                                  PyObject*    obj);
   long          pyVersion       ();
#if defined(DEBUG_PYTHON)
   void          pyAssignGlobal  (char*        name,
                                  PyObject*    obj);
   void          pyDebug         (PyObject*    obj = Py_None);
#endif
public:
   void setEnablePythonSIGINTHandler(bool value);
public:
   class PyGILStateWrapper {
   public:
      PyGILStateWrapper()  { ensure(); }
      ~PyGILStateWrapper() { release(); }
   public:
      void ensure()
      { 
#if defined(PYTHON_SUPPORTS_GIL)
         state = PyGILState_Ensure(); 
#endif
      }
      void release()
      { 
#if defined(PYTHON_SUPPORTS_GIL)
         PyGILState_Release(state);
#endif
      }
   protected:
#if defined(PYTHON_SUPPORTS_GIL)
      PyGILState_STATE  state;
#endif
   };
protected:
   std::list<PyObject*>  xcpts;
protected:
   static bool       initialized;
   static PyObject*  main_dict;
protected:
   virtual void error   (char*  text) = 0;
   virtual void warning (char*  text) = 0;
protected:
   virtual void handleException();
   virtual void exceptionExpected();
   virtual void exceptionError();
};

#endif
