#include "RepoEditorView.h"
#include "RepoEditorModel.h"
#include <QtGui/QHeaderView>
#include <QtCore/QObject>

RepoEditor::View::View(Model*    model,
                       QWidget*  parent)
#if QT_VERSION >= 0x040200
   : HTableView(QStyleOptionViewItem::Left, 1, INT_MAX, parent),
#else
   : HTableView(QStyleOptionViewItem::Bottom, 1, INT_MAX, parent),
#endif
     model(model),
     contextMenu(NULL)
{
   sortedModel = new QSortFilterProxyModel(parent);
   sortedModel->setSourceModel(model);
   setModel(sortedModel);

   setShowGrid(false);
   setGridStyle(Qt::NoPen);
   setAlternatingRowColors(true);
   setContextMenuPolicy(Qt::CustomContextMenu);
   setTabKeyNavigation(false);

   verticalHeader()->hide();
   setHorizontalHeader(new QHeaderView(Qt::Horizontal, this));

   horizontalHeader()->setSelectionBehavior(QAbstractItemView::SelectColumns);
   horizontalHeader()->setStretchLastSection(true);
   horizontalHeader()->setMovable(true);
   horizontalHeader()->setClickable(true);

   connect(horizontalHeader(), SIGNAL(sectionClicked(int)), 
           this, SLOT(handleHorizontalHeaderClicked(int)));

   connect(this, SIGNAL(clicked(const QModelIndex&)),
           this, SLOT(handleActivated(const QModelIndex&)));
   connect(this, SIGNAL(customContextMenuRequested(const QPoint&)),
           this, SLOT(handleCustomContextMenuRequested(const QPoint&)));

   resizeColumnsToContents();
}

void  RepoEditor::View::handleHorizontalHeaderClicked(int  column)
{
   sortByColumn(column);
   sortedModel->setFilterKeyColumn(column);
}

void  RepoEditor::View::handleActivated(const QModelIndex&  ix)
{
   if (ix.column() == USE) {
      model->toggleUse(sortedModel->mapToSource(ix));
   }
}

void  RepoEditor::View::handleCustomContextMenuRequested(const QPoint&  pos)
{
   QModelIndex  ix = sortedModel->mapToSource(indexAt(pos));

   if (ix.column() == USE) {
      // DO NOTHING
   } else {
      if (contextMenu == NULL) {
         contextMenu = new ContextMenu(model, this);
      }
      contextMenu->exec(ix, viewport()->mapToGlobal(pos));
   }
}

int  RepoEditor::View::currentRow()
{
   return sortedModel->mapToSource(currentIndex()).row();
}

void  RepoEditor::View::setCurrentRow (int  row)
{
   setCurrentIndex(sortedModel->mapFromSource(model->createIndex(row)));
}

QModelIndexList  RepoEditor::View::selectedRows() const
{
   QModelIndexList            result = HTableView::selectedRows();
   QModelIndexList::iterator  it     = result.begin();

   for (; it != result.end(); ++it) {
      *it = sortedModel->mapToSource(*it);
   }

   return result;
}

void  RepoEditor::View::currentChanged(const QModelIndex&  current,
                                       const QModelIndex&  previous)
{
   int  currentRow  = sortedModel->mapToSource(current).row();
   int  previousRow = sortedModel->mapToSource(previous).row();
   if (currentRow != previousRow) {
      emit currentChanged(currentRow, previousRow);
   }
}
